package org.jeecg.modules.fbsBasedate.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.fbsBasedate.entity.FbsTechnologicalProcess;
import org.jeecg.modules.fbsBasedate.entity.FbsTechnologicalProcessItem;
import org.jeecg.modules.fbsBasedate.service.IFbsTechnologicalProcessService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 工艺流程档案
 * @author： jeecg-boot
 * @date：   2021-01-14
 * @version： V1.0
 */
@RestController
@RequestMapping("/fbsBasedate/fbsTechnologicalProcess")
@Slf4j
public class FbsTechnologicalProcessController {
	@Autowired
	private IFbsTechnologicalProcessService fbsTechnologicalProcessService;
	
	/**
	  * 分页列表查询
	 * @param fbsTechnologicalProcess
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FbsTechnologicalProcess>> queryPageList(FbsTechnologicalProcess fbsTechnologicalProcess,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FbsTechnologicalProcess>> result = new Result<IPage<FbsTechnologicalProcess>>();
		QueryWrapper<FbsTechnologicalProcess> queryWrapper = QueryGenerator.initQueryWrapper(fbsTechnologicalProcess, req.getParameterMap());
		queryWrapper.lambda().eq(FbsTechnologicalProcess::getDelFlag,"0");
		Page<FbsTechnologicalProcess> page = new Page<FbsTechnologicalProcess>(pageNo, pageSize);
		IPage<FbsTechnologicalProcess> pageList = fbsTechnologicalProcessService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fbsTechnologicalProcess
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FbsTechnologicalProcess> add(@RequestBody FbsTechnologicalProcess fbsTechnologicalProcess) {
		Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		try {
			fbsTechnologicalProcessService.save(fbsTechnologicalProcess);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fbsTechnologicalProcess
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FbsTechnologicalProcess> edit(@RequestBody FbsTechnologicalProcess fbsTechnologicalProcess) {
		Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		FbsTechnologicalProcess fbsTechnologicalProcessEntity = fbsTechnologicalProcessService.getById(fbsTechnologicalProcess.getId());
		if(fbsTechnologicalProcessEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsTechnologicalProcessService.updateById(fbsTechnologicalProcess);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FbsTechnologicalProcess> delete(@RequestParam(name="id",required=true) String id) {
		Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		FbsTechnologicalProcess fbsTechnologicalProcess = fbsTechnologicalProcessService.getById(id);
		if(fbsTechnologicalProcess==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsTechnologicalProcessService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FbsTechnologicalProcess> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fbsTechnologicalProcessService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FbsTechnologicalProcess> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		FbsTechnologicalProcess fbsTechnologicalProcess = fbsTechnologicalProcessService.getById(id);
		if(fbsTechnologicalProcess==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fbsTechnologicalProcess);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FbsTechnologicalProcess> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FbsTechnologicalProcess fbsTechnologicalProcess = JSON.parseObject(deString, FbsTechnologicalProcess.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fbsTechnologicalProcess, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FbsTechnologicalProcess> pageList = fbsTechnologicalProcessService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "工艺流程档案列表");
      mv.addObject(NormalExcelConstants.CLASS, FbsTechnologicalProcess.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("工艺流程档案列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FbsTechnologicalProcess> listFbsTechnologicalProcesss = ExcelImportUtil.importExcel(file.getInputStream(), FbsTechnologicalProcess.class, params);
              for (FbsTechnologicalProcess fbsTechnologicalProcessExcel : listFbsTechnologicalProcesss) {
                  fbsTechnologicalProcessService.save(fbsTechnologicalProcessExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFbsTechnologicalProcesss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

	/**
	* @Author chenchuang
	* @Description //TODO 添加主表和子表明细
	* @Date 2021/1/16 13:57
	* @Param [fbsTechnologicalProcess]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.fbsBasedate.entity.FbsTechnologicalProcess>
	*/
	 @PostMapping(value = "/addAll")
	 public Result<FbsTechnologicalProcess> addAll(@RequestBody FbsTechnologicalProcess fbsTechnologicalProcess) {
		 Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		 try {
			 fbsTechnologicalProcessService.addAll(fbsTechnologicalProcess);
			 result.success("添加成功！");
		 } catch (Exception e) {
			 e.printStackTrace();
			 log.info(e.getMessage());
			 result.error500("操作失败");
		 }
		 return result;
	 }

	/**
	* @Author chenchuang
	* @Description //TODO 修改全部
	* @Date 2021/1/16 14:17
	* @Param [fbsTechnologicalProcess]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.fbsBasedate.entity.FbsTechnologicalProcess>
	*/
	 @PutMapping(value = "/editAll")
	 public Result<FbsTechnologicalProcess> editAll(@RequestBody FbsTechnologicalProcess fbsTechnologicalProcess) {
		 Result<FbsTechnologicalProcess> result = new Result<FbsTechnologicalProcess>();
		 FbsTechnologicalProcess fbsTechnologicalProcessEntity = fbsTechnologicalProcessService.getById(fbsTechnologicalProcess.getId());
		 if(fbsTechnologicalProcessEntity==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 fbsTechnologicalProcessService.updateAll(fbsTechnologicalProcess);
				 result.success("修改成功！");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("修改失败："+e.getMessage());
			 }

		 }
		 return result;
	 }
}
